@@ -1,7 +1,4 @@
 #########################################################################
-# $Date$
-# $Revision$
-# $Author$
 # ex: set ts=8 sw=4 et
 #########################################################################
 package WWW::Facebook::API;
@@ -10,7 +7,7 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.4.13');
+use version; our $VERSION = qv('0.4.17');
 
 use LWP::UserAgent;
 use Time::HiRes qw(time);
@@ -19,12 +16,19 @@ use Encode qw(encode_utf8 is_utf8);
 use CGI;
 use CGI::Util qw(escape);
 
+use WWW::Facebook::API::Exception;
+
 our @namespaces = qw(
-    Auth            Canvas          Data
-    Events          FBML            Feed
-    FQL             Friends         Groups
-    Notifications   Photos          Profile
-    Users           Marketplace     Pages
+    Admin           Application     Auth
+    Canvas          Comments        Connect
+    Data            Events          FBML
+    Feed            FQL             Friends
+    Groups          Intl            Links
+    LiveMessage     Message         Notes
+    Notifications   Pages           Permissions
+    Photos          Profile         SMS
+    Status          Stream          Users
+    Video
 );
 
 for (@namespaces) {
@@ -74,7 +78,7 @@ our %attributes = (
             popup               next        session_key
             session_expires     session_uid callback
             app_path            ua          query
-            config              app_id
+            config              app_id      call_as_apikey
             )
     ),
 );
@@ -195,23 +199,28 @@ sub call_success {
 
 sub call {
     my ( $self, $method, %args ) = @_;
-    my ( $response, $params, $sig, $img_data );
+    my ( $response, $params, $sig, $raw_data, $filename );
     $self->call_success(1);
 
-    ( $params, $img_data ) =
+    if ( $self->call_as_apikey ) { 
+         $args{'call_as_apikey'} = $self->call_as_apikey;
+    }
+
+    ( $params, $raw_data, $filename ) =
         $self->_format_and_check_params( $method, %args );
     $sig = $self->generate_sig(
         params => $params,
         secret => delete $params->{'secret'},
     );
 
-    $response = $self->_post_request( $params, $sig, $img_data );
+    $response = $self->_post_request( $params, $sig, $raw_data, $filename );
 
     carp $self->log_string( $params, $response ) if $self->debug;
     if ( $self->_has_error_response($response) ) {
         if ( $self->throw_errors ) {
-            confess "Error during REST $method call:",
-                $self->log_string( $params, $response );
+            warn WWW::Facebook::API::Exception->new(
+                $method, $params, $response
+            );
         }
     }
 
@@ -427,7 +436,7 @@ sub _format_and_check_params {
     croak '_format_and_check_params must be called in list context!'
         if !wantarray;
 
-    return ( $params, delete $params->{'data'} );
+    return ( $params, delete $params->{'data'}, delete $params->{'filename'} );
 }
 
 sub _has_error_response {
@@ -482,16 +491,16 @@ sub _is_empty_response {
 }
 
 sub _post_request {
-    my ( $self, $params, $sig, $img_data ) = @_;
+    my ( $self, $params, $sig, $raw_data, $filename ) = @_;
 
     my $post_params = [ map { $_ => $params->{$_} } sort keys %{$params} ];
     push @{$post_params}, 'sig' => $sig;
 
-    if ($img_data) {
+    if ($raw_data) {
         push @{$post_params}, data => [
-            undef, 'filename',
-            'Content-Type' => 'image/jpeg',
-            'Content'      => $img_data,
+            undef, ( $filename || 'filename'),
+            'Content-Type' => 'application/octet-stream',
+            'Content'      => $raw_data,
         ];
     }
     return $self->ua->post(
@@ -510,7 +519,8 @@ WWW::Facebook::API - Facebook API implementation
 
 =head1 VERSION
 
-This document describes WWW::Facebook::API version 0.4.13
+This document and others distributed with this module describe
+WWW::Facebook::API version 0.4.17
 
 =head1 SYNOPSIS
 
@@ -613,6 +623,14 @@ All method names from the Facebook API are lower_cased instead of CamelCase.
 
 =over
 
+=item admin
+
+admin namespace of the API (See L<WWW::Facebook::API::Admin>).
+
+=item application
+
+application namespace of the API (See L<WWW::Facebook::API::Application>).
+
 =item auth
 
 For desktop apps, these are synonymous:
@@ -648,6 +666,14 @@ Work with the canvas. See L<WWW::Facebook::API::Canvas>.
     $response = $client->canvas->in_fb_canvas( $q )
     $response = $client->canvas->in_frame( $q )
 
+=item comments
+
+comments namespace of the API (See L<WWW::Facebook::API::Comments>).
+
+=item connect
+
+connect namespace of the API (See L<WWW::Facebook::API::Connect>).
+
 =item data
 
 data namespace of the API (See L<WWW::Facebook::API::Data>).
@@ -682,13 +708,6 @@ All method names from the Facebook API are lower_cased instead of CamelCase:
     $response = $client->fbml->refresh_img_src( url => '');
     $response = $client->fbml->refresh_ref_url( url => '');
 
-=item fql
-
-fql namespace of the API (See L<WWW::Facebook::API::FQL>):
-
-    $response = $client->fql->query( query => 'FQL query' );
-
-
 =item feed
 
 feed namespace of the API (See L<WWW::Facebook::API::Feed>).
@@ -713,6 +732,12 @@ All method names from the Facebook API are lower_cased instead of CamelCase:
             ...
     );
 
+=item fql
+
+fql namespace of the API (See L<WWW::Facebook::API::FQL>):
+
+    $response = $client->fql->query( query => 'FQL query' );
+
 =item friends
 
 friends namespace of the API (See L<WWW::Facebook::API::Friends>).
@@ -732,6 +757,26 @@ All method names from the Facebook API are lower_cased instead of CamelCase:
     $response = $client->groups->get( uid => 234324, gids => [2423,334] );
     $response = $client->groups->get_members( gid => 32 );
 
+=item intl
+
+intl namespace of the API (See L<WWW::Facebook::API::Intl>).
+
+=item links
+
+links namespace of the API (See L<WWW::Facebook::API::Links>).
+
+=item livemesssage
+
+liveMesssage namespace of the API (See L<WWW::Facebook::API::LiveMesssage>).
+
+=item message
+
+message namespace of the API (See L<WWW::Facebook::API::Message>).
+
+=item notes
+
+notes namespace of the API (See L<WWW::Facebook::API::Notes>).
+
 =item notifications
 
 notifications namespace of the API (See L<WWW::Facebook::API::Notifications>).
@@ -749,37 +794,9 @@ All method names from the Facebook API are lower_cased instead of CamelCase:
         fbml  => 'fbml version of email body',
     );
 
-=item marketplace
-
-marketplace namespace of the API (See L<WWW::Facebook::API::Marketplace>).
-All method names from the Facebook API are lower_cased instead of CamelCase:
-
-    $categories = $client->marketplace->get_categories;
-    $subcats = $client->marketplace->get_subcategories(
-        category => 'category',
-    );
-    $listings_response = $client->marketplace->get_listings(
-        listing_ids => [@listing_ids],
-        uids => [@uids],
-    );
-    $response = $client->marketplace->search(
-        category => 'category',
-        subcategory => 'subcategory',
-        query => 'query',
-    );
-    $listing_id = $client->marketplace->create_listing(
-        listing_id => 0|existing_id,
-        show_on_profile => 0|1,
-        listing_attrs => 'JSON',
-    );
-    $success = $client->marketplace->remove_listing(
-        listing_id => 'id',
-        status => 'SUCCESS|NOT_SUCCESS|DEFAULT',
-    );
-
 =item pages
 
-pages namespace of the API (See L<WWW::Facebook::API::Marketplace>). All
+pages namespace of the API (See L<WWW::Facebook::API::Pages>). All
 method names from the Facebook API are lower_cased instead of CamelCase:
 
     $response = $client->pages->get_info(
@@ -792,6 +809,10 @@ method names from the Facebook API are lower_cased instead of CamelCase:
     $is_admin = $client->pages->is_admin( page_id => 'page' );
     $is_fan = $client->pages->is_fan( page_id => 'page', uid => 'uid' )
 
+=item permissions
+
+permissions namespace of the API (See L<WWW::Facebook::API::Permissions>).
+
 =item photos
 
 photos namespace of the API (See L<WWW::Facebook::API::Photos>).
@@ -830,6 +851,18 @@ All method names from the Facebook API are lower_cased instead of CamelCase:
         mobile_profile => 'markup',
     );
 
+=item sms
+
+sms namespace of the API (See L<WWW::Facebook::API::SMS>).
+
+=item status
+
+status namespace of the API (See L<WWW::Facebook::API::Status>).
+
+=item stream
+
+stream namespace of the API (See L<WWW::Facebook::API::Stream>).
+
 =item users
 
 users namespace of the API (See L<WWW::Facebook::API::Users>).
@@ -843,12 +876,16 @@ All method names from the Facebook API are lower_cased instead of CamelCase:
     $response = $client->users->has_app_permission(
         ext_perm => 'status_update|photo_upload'
     );
-    $app_added = $client->users->is_app_added;
+    $app_added = $client->users->is_app_user;
     $response = $client->users->set_status(
         status => 'status message',
         clear => 1|0,
     );
 
+=item video
+
+video namespace of the API (See L<WWW::Facebook::API::Video>).
+
 =back
 
 =head1 ATTRIBUTE METHODS
@@ -1027,7 +1064,7 @@ when an error is returned from the REST server.
 =item ua
 
 The L<LWP::UserAgent> agent used to communicate with the REST server.
-The agent_alias is initially set to "Perl-WWW-Facebook-API/0.4.13".
+The agent_alias is initially set to "Perl-WWW-Facebook-API/0.4.17".
 
 =back
 
@@ -1201,10 +1238,10 @@ set. Uses the defaults for those values that are needed and not supplied.
 
 Format method parameters (given in C<%args>) according to Facebook API
 specification. Returns a list of items: A hash reference of the newly
-formatted params (based on C<%params>) and the image data if the call is an
-photo upload:
+formatted params (based on C<%params>) and the raw data (and filename, if
+passed in) if the call is a photo or video upload:
 
-    ($params, $img_data) = $self->_format_and_check_params( $method, %args );
+    ($params, $raw_data, $filename) = $self->_format_and_check_params( $method, %args );
 
 =item _has_error_response( $response )
 
@@ -1216,10 +1253,11 @@ true if response is an error, false otherwise.
 Determines if the response is an empty hash or array reference. Returns true
 if the response is empty, false otherwise.
 
-=item _post_request( $params_hashref, $sig, $img_data )
+=item _post_request( $params_hashref, $sig, $raw_data, $filename )
 
 Used by C<call> to post the request to the REST server and return the
-response. C<$img_data> is used when uploading an photo to Facebook.
+response. C<$raw_data> and C<$filename> are used when uploading a photo or
+video to Facebook.
 
 =item _parse($string)
 
@@ -1372,16 +1410,26 @@ David Romano  C<< <unobe@cpan.org> >>
 
 =head1 CONTRIBUTORS
 
+Anthony Bouvier C<< none >>
+
 Clayton Scott C<< http://www.matrix.ca >>
 
 David Leadbeater C<< http://dgl.cx >>
 
+Derek Del Conte C<< derek@delconte.org >>
+
 Gisle Aas C<< none >>
 
 J. Shirley C<< <jshirley@gmail.com> >>
 
 Jim Spath C<< <jspath@gmail.com> >>
 
+Kevin Riggle C<< none >>
+
+Larry Mak C<< none >>
+
+Louis-Philippe C<< none >>
+
 Matt Sickler C<< <imMute@mail.msk3.ath.cx> >>
 
 Nick Gerakines C<< <nick@socklabs.com> >>
@@ -1402,15 +1450,9 @@ Skyler Clark C<< none >>
 
 Thomas Sibley C<< <tsibley@cpan.org> >>
 
-Derek Del Conte C<< derek@delconte.org >>
-
-King Mak C<< none >>
-
-Louis-Philippe C<< none >>
-
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2007, David Romano C<< <unobe@cpan.org> >>. All rights reserved.
+Copyright (c) 2007-2009, David Romano C<< <unobe@cpan.org> >>. All rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
