@@ -1,7 +1,4 @@
 #######################################################################
-# $Date$
-# $Revision$
-# $Author$
 # ex: set ts=8 sw=4 et
 #########################################################################
 package WWW::Facebook::API::Canvas;
@@ -9,22 +6,41 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.4.13');
-
 sub get_fb_params {
     my $self = shift;
     $self->base->query(shift);
 
     my $fb_params = {};
-    my @query = grep {m/^fb_sig_/xms} $self->base->query->param;
+	# verify - FB Connect or Native?
+	my @query = '';
+	my $api_key_pre = $self->base->api_key."_";
+	if ($self->base->query->param('fb_sig')) {
+    	@query = grep {m/^fb_sig_/xms} $self->base->query->param;
+	}
+	else {
+		@query = grep {m/^$api_key_pre/xms} $self->base->query->cookie;
+	}
+
     for my $param (@query) {
-        my @values = $self->base->query->param($param);
+		my @values = '';
+		if ($self->base->query->param('fb_sig')) {
+       		@values = $self->base->query->param($param);
+		}
+		else {
+			@values = $self->base->query->cookie($param)->value;
+		}
         if ( @values > 1 || ref $values[0] ) {
             croak "Multiple values for $param: Are you using POST for forms?";
         }
 
-        my $attribute = ( $param =~ /^fb_sig_ (.*) $/xms )[0];
-        $fb_params->{$attribute} = $self->base->query->param($param);
+		if ($self->base->query->param('fb_sig')) {
+        	my $attribute = ( $param =~ /^fb_sig_ (.*) $/xms )[0];
+        	$fb_params->{$attribute} = $self->base->query->param($param);
+		}
+		else {
+        	my $attribute = ( $param =~ /^$api_key_pre (.*) $/xms )[0];
+        	$fb_params->{$attribute} = $self->base->query->cookie($param)->value;
+		}
     }
 
     return $fb_params;
@@ -53,11 +69,11 @@ sub validate_sig {
     $self->base->query(shift);
 
     my $fb_params = $self->get_fb_params;
-    return unless $self->base->query->param('fb_sig');
+    return unless $self->base->query->param('fb_sig') || $self->base->query->cookie('fbsetting_'.$self->base->api_key)->value;
     return $fb_params
         if $self->base->verify_sig(
         params => $fb_params,
-        sig    => $self->base->query->param('fb_sig'),
+        sig    => $self->base->query->param('fb_sig') || $self->base->query->cookie('fbsetting_'.$self->base->api_key)->value,
         );
 
     return;
@@ -97,10 +113,6 @@ __END__
 
 WWW::Facebook::API::Canvas - Facebook Canvas
 
-=head1 VERSION
-
-This document describes WWW::Facebook::API::Canvas version 0.4.13
-
 =head1 SYNOPSIS
 
     use WWW::Facebook::API;
@@ -220,15 +232,16 @@ Please report any bugs or feature requests to
 C<bug-www-facebook-api@rt.cpan.org>, or through the web interface at
 L<http://rt.cpan.org>.
 
-=head1 AUTHOR
+=head1 AUTHORS
 
 David Leadbeater  C<< http://dgl.cx >>
+
 David Romano  C<< <unobe@cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
 Copyright (c) 2007, David Leadbeater C<< http://dgl.cx >>.
-David Romano C<< <unobe@cpan.org> >>. All rights reserved. 
+Certain parts copyright (c) 2007-2009, David Romano C<< <unobe@cpan.org> >>. All rights reserved. 
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
